/* eslint-disable no-undef */
// 注册名为 imageUpload 的插件
CKEDITOR.plugins.add('imageUpload', {
  init: function(editor) {
    // 添加工具栏按钮
    editor.ui.addButton('imageUpload', {
      icon: `${this.path}icon.png`,
      label: 'upload',
      command: 'imageUpload'
    });
    // 点击按钮时执行：选择本地图片
    editor.addCommand('imageUpload', {
      exec: function() {
        const $fileInput = document.createElement('input');
        $fileInput.type = 'file';
        $fileInput.accept = 'image/*';
        $fileInput.click();
        $fileInput.addEventListener('change', e => {
          const file = $fileInput.files[0];
          getUploadUrl(file);
        });
      }
    });
    // 粘贴时执行：拦截图片的默认操作
    editor.on('paste', e => {
      const file = e.data.dataTransfer.getFile(0);
      if (file) {
        e.stop();
        getUploadUrl(file);
      }
    });

    // 获取上传地址
    async function getUploadUrl(file) {
      const { uid, noteId } = await mxChromeApi.getUserProfile();
      const urlOptions = {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({
          type: 'noteres',
          filename: `${uid}_${noteId}_${Date.now()}`,
          uid
        })
      };
      console.log('getUploadUrl options', urlOptions);
      const urlres = await fetch('https://syncapi.maxthon.com/userfile/getUploadUrl?key=_passverify', urlOptions);
      const urlRet = await urlres.json();
      console.log('getUploadUrl result', urlRet);
      readImg(file, urlRet);
    }

    // 读取图片
    function readImg(file, urlRet) {
      const fileReader = new FileReader();
      fileReader.readAsDataURL(file);
      fileReader.onload = function() {
        const base64 = this.result.split(',').pop(); // await file.arrayBuffer()
        uploadImg(file, urlRet, base64);
      };
    }

    // 上传图片
    function uploadImg(file, urlRet, base64) {
      const uploadOptios = {
        url: urlRet.uploadUrl,
        method: 'POST',
        type: 'file',
        file_ext: file.type.split('/').pop(),
        headers: {
          'Authorization': urlRet.token,
          'X-Bz-File-Name': urlRet.filename
        },
        body: base64
      };
      console.log('upload options', uploadOptios);
      maxthon.send(uploadRes => setEditor(file, urlRet, uploadRes), { from: 'jsController', cmd: 'fetch', param: uploadOptios });
    }

    // 插入编辑器
    function setEditor(file, urlRet, uploadRes) {
      const uploadRet = JSON.parse(uploadRes);
      console.log('upload result', uploadRet);
      if (uploadRet.code || uploadRet.values.code) return;
      const url = `mxn://r?u=${encodeURIComponent(urlRet.url)}`;
      editor.insertHtml(`<img src="${url}" alt="${file.name}" data-size="${uploadRet.values.contentLength}"  />`);
    }
  }
});
